@@ -29,10 +29,9 @@ struct Unit;
 class CompilationUnitMapper
 {
 public:
-    ~CompilationUnitMapper();
-
     CompiledData::Unit *get(
             const QString &cacheFilePath, const QDateTime &sourceTimeStamp, QString *errorString);
+    static void invalidate(const QString &cacheFilePath);
 
 private:
     CompiledData::Unit *open(
